function printopf(bus, gen, branch, f, success, et, fd, pretty, ugly)
%PRINTOPF   Prints optimal power flow results.
%   printopf(bus, gen, branch, f, success, et, fd, pretty, ugly) prints opf
%   results to fd (a file descriptor). If pretty is true it formats things
%   nicely with headings and stuff.  If ugly is true it prints some raw data
%   for our Perl database interface code. The objective function value is
%   given in f and the elapsed time (seconds to compute opf) in et.

%   MATPOWER Version 1.0
%   by Ray Zimmerman, PSERC Cornell    9/16/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialization -----
%% default arguments
if nargin < 9
	ugly = 0;			%% don't print the ugly stuff
	if nargin < 8
		pretty = 1;		%% print the pretty stuff
		if nargin < 7
			fd = 1;		%% print to stdio by default
		end
	end
end

%% define named indices into bus, gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% constant
j = sqrt(-1);

%% parameters
nb = size(bus, 1);
nl = size(branch, 1);
ng = size(gen, 1);

%%----- print the stuff -----
if success
	if pretty
		%% elapsed time
		fprintf(fd, '\nConverged in %.2f seconds', et);

		%% objective function value
		fprintf(fd, '\n===  Objective Function Value  ===');
		fprintf(fd, '\nf = %.2f $/hr', f);
		fprintf(fd, '\n');
		
% 		%% total pay to generators
% 		e2i = zeros(max(bus(:, BUS_I)), 1);		%% need internal bus numbering for a second
% 		e2i(bus(:, BUS_I)) = [1:size(bus, 1)]';
% 		genprice = bus(e2i(gen(:, GEN_BUS)), LAM_P) + gen(:, MU_PMIN);
% 		pay = genprice .* gen(:, PG);
% 		fprintf(fd, '\n=======  Pay to Generators  =======');
% 		fprintf(fd, '\nBus   Price   *    Pg    =   Pay');
% 		fprintf(fd, '\n #    $/MWh        MW        $/hr');
% 		fprintf(fd, '\n---  --------   --------   --------');
% 		for i = 1:ng
% 			if gen(i, PG)
% 				fprintf(fd, '\n%3d%9.3f  *%8.2f  =%8.2f', gen(i, GEN_BUS), genprice(i), gen(i, PG), pay(i));
% 			else
% 				fprintf(fd, '\n%3d     -           -         -   ', gen(i, GEN_BUS));
% 			end
% 		end
% 		fprintf(fd, '\n         Total:%8.2f   %8.2f', sum(gen(:, PG)), sum(pay));
% 		fprintf(fd, '\n');
			
		%% bus data
		fprintf(fd, '\n====================================  Bus Data  ====================================');
		fprintf(fd, '\nBus         Voltage            Generation             Load          Lamda ($/MVA.hr)');
		fprintf(fd, '\n #    Mag (pu)   Ang (deg)   P (MW)   Q (MVAR)   P (MW)   Q (MVAR)     P        Q');
		fprintf(fd, '\n---  ----------  ---------  --------  --------  --------  --------  -------  -------');
		for i = 1:nb
			fprintf(fd, '\n%3d%10.4f%11.3f', bus(i, [BUS_I, VM, VA]));
			g = find(gen(:, GEN_BUS) == bus(i, BUS_I));
			if gen(g, GEN_STATUS)
				fprintf(fd, '%11.2f%10.2f', gen(g, [PG, QG]));
			else
				fprintf(fd, '        -         -  ');
			end
			if bus(i, PD) | bus(i, QD)
				fprintf(fd, '%10.2f%10.2f', bus(i, [PD, QD]));
			else
				fprintf(fd, '       -         -  ');
			end
			fprintf(fd, '%10.3f', bus(i, LAM_P));
			if abs(bus(i, LAM_Q)) > 1e-8
				fprintf(fd, '%8.3f', bus(i, LAM_Q));
			else
				fprintf(fd, '     -   ');
			end
		end
		fprintf(fd, '\n');
		
		%% branch data
		fprintf(fd, '\n================  Branch Data  =====================');
		fprintf(fd, '\n From   To        Avg. Flow             Loss');
		fprintf(fd, '\n Bus    Bus    P (MW)   Q (MVAR)   P (MW)   Q (MVAR)');
		fprintf(fd, '\n-----  -----  --------  --------  --------  --------');
		fprintf(fd, '\n%4d%7d%10.2f%10.2f%10.3f%10.2f', ...
				[	branch(:, [F_BUS, T_BUS]), ...
					(branch(:, [PF, QF]) - branch(:, [PT, QT]))/2, ...
					branch(:, [PF, QF]) + branch(:, [PT, QT]) ...
				]');
		fprintf(fd, '\n');
		
		%% constraint data
		%% voltage constraints
		fprintf(fd, '\n==========  Voltage Constraints  =========');
		fprintf(fd, '\nBus');
		fprintf(fd, '\n #   Vmin mu  Vmin    |V|   Vmax   Vmax mu');
		fprintf(fd, '\n---  -------  -----  -----  -----  -------');
		for i = 1:nb
			fprintf(fd, '\n%3d', bus(i, BUS_I));
			if bus(i, MU_VMIN) > 1e-8
				fprintf(fd, '%8.3f', bus(i, MU_VMIN));
			else
				fprintf(fd, '     -  ');
			end
			fprintf(fd, '%8.3f%7.3f%7.3f', bus(i, [VMIN, VM, VMAX]));
			if bus(i, MU_VMAX) > 1e-8
				fprintf(fd, '%8.3f', bus(i, MU_VMAX));
			else
				fprintf(fd, '     -   ');
			end
		end
		fprintf(fd, '\n');
		
		%% generator constraints
		fprintf(fd, '\n============  Generation Constraints  ==========');
		fprintf(fd, '\nBus             Active Power Limits');
		fprintf(fd, '\n #   Pmin mu    Pmin       P        Pmax    Pmax mu');
		fprintf(fd, '\n---  -------  --------  --------  --------  -------');
		for i = 1:ng
			fprintf(fd, '\n%3d', gen(i, GEN_BUS));
			if gen(i, MU_PMIN) > 1e-8
				fprintf(fd, '%8.3f', gen(i, MU_PMIN));
			else
				fprintf(fd, '     -  ');
			end
			if gen(i, PG)
				fprintf(fd, '%10.2f%10.2f%10.2f', gen(i, [PMIN, PG, PMAX]));
			else
				fprintf(fd, '%10.2f      -   %10.2f', gen(i, [PMIN, PMAX]));
			end
			if gen(i, MU_PMAX) > 1e-8
				fprintf(fd, '%9.3f', gen(i, MU_PMAX));
			else
				fprintf(fd, '      -  ');
			end
		end
		fprintf(fd, '\n');
		fprintf(fd, '\nBus            Reactive Power Limits');
		fprintf(fd, '\n #   Qmin mu    Qmin       Q        Qmax    Qmax mu');
		fprintf(fd, '\n---  -------  --------  --------  --------  -------');
		for i = 1:ng
			fprintf(fd, '\n%3d', gen(i, GEN_BUS));
			if gen(i, MU_QMIN) > 1e-8
				fprintf(fd, '%8.3f', gen(i, MU_QMIN));
			else
				fprintf(fd, '     -  ');
			end
			fprintf(fd, '%10.2f%10.2f%10.2f', gen(i, [QMIN, QG, QMAX]));
			if gen(i, MU_QMAX) > 1e-8
				fprintf(fd, '%9.3f', gen(i, MU_QMAX));
			else
				fprintf(fd, '      -  ');
			end
		end
		fprintf(fd, '\n');
		
		%% line flow constraints
		fprintf(fd, '\n=================  Line Flow Constraints  ==================');
		fprintf(fd, '\n From     "From" End        Limit       "To" End        To');
		fprintf(fd, '\n Bus   |Sf| mu    |Sf|     |Smax|     |St|    |St| mu   Bus');
		fprintf(fd, '\n-----  -------  --------  --------  --------  -------  -----');
		for i = 1:nl
			fprintf(fd, '\n%4d', branch(i, F_BUS));
			if branch(i, MU_SF) > 1e-8
				fprintf(fd, '%10.3f', branch(i, MU_SF));
			else
				fprintf(fd, '      -   ');
			end
			fprintf(fd, '%9.2f%10.2f%10.2f', ...
				[abs(branch(i, PF) + j * branch(i, QF)), ...
				branch(i, RATE_A), abs(branch(i, PT) + j * branch(i, QT))]);
			if branch(i, MU_ST) > 1e-8
				fprintf(fd, '%10.3f', branch(i, MU_ST));
			else
				fprintf(fd, '      -   ');
			end
			fprintf(fd, '%6d', branch(i, T_BUS));
		end
		fprintf(fd, '\n');
	end
	
	%% print raw data for Perl database interface
	if ugly
		fprintf(fd, '----------  raw PS::Soln data below  ----------\n');
		fprintf(fd, 'bus\n');
		fprintf(fd, '%d\t%d\t%g\t%g\t%g\t%g\t%g\t%g\n', ...
					bus(:, [BUS_I, BUS_TYPE, VM, VA, LAM_P, LAM_Q, MU_VMAX, MU_VMIN])');
	
		fprintf(fd, 'branch\n');
		fprintf(fd, '%d\t%g\t%g\t%g\t%g\t%g\t%g\n', ...
					[[1:nl]' branch(:, [PF, QF, PT, QT, MU_SF, MU_ST])]');
	
		fprintf(fd, 'gen\n');
		fprintf(fd, '%d\t%g\t%g\t%g\t%d\t%g\t%g\t%g\t%g\n', ...
					[[1:ng]' gen(:, [PG, QG, VG, GEN_STATUS, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN])]');
		fprintf(fd, 'end\n');
		fprintf(fd, '----------  raw PS::Soln data above  ----------\n');
	end
else
	fprintf(fd, 'did not converge\n');
end

return;
